/*
  Original code created by DFRobot for their probes, 
  adapted by haze5@icloud.com Mysensors for the project.
*/

#include <SPI.h>
#include <MySensor.h>
#include "Vcc.h"        // https://github.com/Yveaux/Arduino_Vcc
#include <OneWire.h>

static const float VccMin = 0.0;        // Minimum expected Vcc level, in Volts. (0.6V for 1 AA Alkaline)
static const float VccMax = 3.3;        // Maximum expected Vcc level, in Volts. (1.5V for 1 AA Alkaline)
static const float VccCorrection = 3.29 / 3.31;  // Measured Vcc by multimeter divided by reported Vcc
Vcc vcc(VccCorrection);

#define CHILD_ID_EC 0
#define CHILD_ID_TEMP 1


#define ArrayLenth 10               // times of collection

#define EC_SENSOR_ANALOG_PIN A0         // pH meter Analog output to Arduino Analog Input 0
#define LED_DIGITAL_PIN 13
#define StartConvert 0
#define ReadTemperature 1

unsigned long lastSend = 0;
static const unsigned long SEND_FREQUENCY = 30000;  // Minimum time between send (in milliseconds)


const byte numReadings = 20;     //the number of sample times
byte ECsensorPin = A0;  //EC Meter analog output,pin on analog 1
byte DS18B20_Pin = 3; //DS18B20 signal, pin on digital 2
unsigned int AnalogSampleInterval=25,printInterval=700,tempSampleInterval=850;  //analog sample interval;serial print interval;temperature sample interval
unsigned int readings[numReadings];      // the readings from the analog input
byte index = 0;                  // the index of the current reading
unsigned long AnalogValueTotal = 0;                  // the running total
unsigned int AnalogAverage = 0,averageVoltage=0;                // the average
unsigned long AnalogSampleTime,printTime,tempSampleTime;
float temperature,ECcurrent;

OneWire ds(DS18B20_Pin);  // on digital pin 2

MySensor gw;

float lastEcValue;
static const float deltaEcValue = 0.5;

MyMessage msgEC(CHILD_ID_EC, V_VAR1);
MyMessage msgTemp(CHILD_ID_TEMP, V_TEMP);

void setup()
{

    gw.begin();
    
    pinMode(LED_DIGITAL_PIN, OUTPUT);
    
    // Send the Sketch Version Information to the Gateway
    gw.sendSketchInfo("ECmeter/Temp", "1.0");
    
    // Register all sensors to gw (they will be created as child devices)
    gw.present(CHILD_ID_EC, S_WATER);
    gw.present(CHILD_ID_TEMP, S_TEMP);
    
    // initialize all the readings to 0:
    for (byte thisReading = 0; thisReading < numReadings; thisReading++)
        readings[thisReading] = 0;
    TempProcess(StartConvert);   //let the DS18B20 start the convert
    AnalogSampleTime=millis();
    printTime=millis();
    tempSampleTime=millis();
}

void loop()
{
    // By calling process() you route messages in the background
    gw.process();
    
    unsigned long now = millis();
    bool sendTime = now - lastSend > SEND_FREQUENCY;
    if (sendTime)
    {
        lastSend = now;
        
        //    float v = vcc.Read_Volts();
        //    Serial.print("VCC = " );
        //    Serial.print(v);
        //    Serial.println(" Volts" );
        
        int batteryPcnt = (int)vcc.Read_Perc(VccMin, VccMax);
        //    Serial.print("VCC = " );
        //    Serial.print(batteryPcnt);
        //    Serial.println(" %" );
        
        gw.sendBatteryLevel(batteryPcnt);
    }
    
    if(millis()-AnalogSampleTime>=AnalogSampleInterval)
    {
        AnalogSampleTime=millis();
        // subtract the last reading:
        AnalogValueTotal = AnalogValueTotal - readings[index];
        // read from the sensor:
        readings[index] = analogRead(ECsensorPin);
        // add the reading to the total:
        AnalogValueTotal = AnalogValueTotal + readings[index];
        // advance to the next position in the array:
        index = index + 1;
        // if we're at the end of the array...
        if (index >= numReadings)
            // ...wrap around to the beginning:
            index = 0;
        // calculate the average:
        AnalogAverage = AnalogValueTotal / numReadings;
    }
    
    if(millis()-tempSampleTime>=tempSampleInterval)
    {
        tempSampleTime=millis();
        temperature = TempProcess(ReadTemperature);  // read the current temperature from the  DS18B20
        TempProcess(StartConvert);                   //after the reading,start the convert for next reading
    }
    
    if(millis()-printTime>=printInterval)
    {
        printTime=millis();
        averageVoltage=AnalogAverage*(float)5000/1024;
        Serial.print("Analog value:");
        Serial.print(AnalogAverage);   //analog average,from 0 to 1023
        Serial.print("    Voltage:");
        Serial.print(averageVoltage);  //millivolt average,from 0mv to 4995mV
        Serial.print("mV    ");
        Serial.print("temp:");
        Serial.print(temperature);    //current temperature
        Serial.print("^C     EC:");
        //float TempCoefficient=1.0+0.0185*(temperature-25.0);
        float TempCoefficient=1.0+0.0185*(temperature-22.0);    //temperature compensation formula: fFinalResult(25^C) = fFinalResult(current)/(1.0+0.0185*(fTP-25.0));
        float CoefficientVolatge=(float)averageVoltage/TempCoefficient;
        if(CoefficientVolatge<10)Serial.println("No solution!");   //25^C 1413us/cm<-->about 216mv  if the voltage(compensate)<150,that is <1ms/cm,out of the range
        else if(CoefficientVolatge>3300)Serial.println("Out of the range!");  //>20ms/cm,out of the range
        else
        {
            if(CoefficientVolatge<=448)ECcurrent=6.84*CoefficientVolatge-64.32;   //1ms/cm<EC<=3ms/cm
            else if(CoefficientVolatge<=1457)ECcurrent=6.98*CoefficientVolatge-127;  //3ms/cm<EC<=10ms/cm
            else ECcurrent=5.3*CoefficientVolatge+2278;
        
        
        ECcurrent/=1000;    //convert us/cm to ms/cm
        Serial.print(ECcurrent,2);  //two decimal
        Serial.println("ms/cm");
        
    }
}
        
    
    if (sendTime || abs(ECcurrent - lastEcValue) > deltaEcValue)
    {
        Serial.print("    EC:");
        Serial.print(ECcurrent, 2);
        Serial.println(" ");
        
        gw.send(msgEC.set(ECcurrent, 2)); // envoi au reseau avec deux decimales
        gw.send(msgTemp.set(temperature, 1)); // envoi au reseau avec une decimale
        
        digitalWrite(LED_DIGITAL_PIN, digitalRead(LED_DIGITAL_PIN) ^ 1);
        
        lastEcValue = ECcurrent;
    }
    
}

/*
 ch=0,let the DS18B20 start the convert;ch=1,MCU read the current temperature from the DS18B20.
 */
float TempProcess(bool ch)
{
    //returns the temperature from one DS18B20 in DEG Celsius
    static byte data[12];
    static byte addr[8];
    static float TemperatureSum;
    if(!ch){
        if ( !ds.search(addr)) {
            Serial.println("no more sensors on chain, reset search!");
            ds.reset_search();
            return 0;
        }
        if ( OneWire::crc8( addr, 7) != addr[7]) {
            Serial.println("CRC is not valid!");
            return 0;
        }
        if ( addr[0] != 0x10 && addr[0] != 0x28) {
            Serial.print("Device is not recognized!");
            return 0;
        }
        ds.reset();
        ds.select(addr);
        ds.write(0x44,1); // start conversion, with parasite power on at the end
    }
    else{
        byte present = ds.reset();
        ds.select(addr);
        ds.write(0xBE); // Read Scratchpad
        for (int i = 0; i < 9; i++) { // we need 9 bytes
            data[i] = ds.read();
        }
        ds.reset_search();
        byte MSB = data[1];
        byte LSB = data[0];
        float tempRead = ((MSB << 8) | LSB); //using two's compliment
        TemperatureSum = tempRead / 16;
    }
    return TemperatureSum;
}
